import re
from html.parser import HTMLParser as BaseHTMLParser


class HTMLParser(BaseHTMLParser):
    def __init__(self):
        super().__init__(convert_charrefs=True)
        self._text = []

    def handle_data(self, data):
        text = data.strip()
        if len(text) > 0:
            text = re.sub('[ \t\r\n]+', ' ', text)
            self._text.append(text + ' ')

    def handle_starttag(self, tag, attrs):
        if tag == 'p':
            self._text.append('\n\n')
        elif tag == 'br':
            self._text.append('\n')

    def handle_endtag(self, tag):
        if tag == 'p':
            self._text.append('\n\n')

    def text(self):
        return ''.join(self._text).strip()



import os
from django.conf import settings

# Django's User model
AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')

# email address to use to sent email from
DEFAULT_FROM_EMAIL = getattr(settings, 'DEFAULT_FROM_EMAIL')

AWS_REGION_NAME = getattr(settings, 'AWS_REGION_NAME', os.getenv('AWS_REGION_NAME', 'eu-west-1'))
AWS_ACCESS_KEY_ID = getattr(settings, 'AWS_ACCESS_KEY_ID', os.getenv('AWS_ACCESS_KEY_ID'))
AWS_SECRET_ACCESS_KEY = getattr(settings, 'AWS_SECRET_ACCESS_KEY', os.getenv('AWS_SECRET_ACCESS_KEY'))

# mapping of verbose email types, used in get_message model methods
AWS_MAIL_TYPES = getattr(settings, 'AWS_MAIL_TYPES', None)

AWS_SNS_VERIFY_NOTIFICATION = getattr(settings, 'AWS_SNS_VERIFY_NOTIFICATION', True)

AWS_SNS_VERIFY_CERTIFICATE = getattr(settings, 'AWS_SNS_VERIFY_CERTIFICATE', True)

AMAZON_SNS_TOPIC_ARN = getattr(settings, 'AMAZON_SNS_TOPIC_ARN', None)

DEFAULT_CHARSET = getattr(settings, 'DEFAULT_CHARSET', 'utf-8')



"""
Mail backend for handling AWS SES via boto3.
Inspired by django-amazon-ses.
"""
import boto3
from botocore.exceptions import ClientError, BotoCoreError

from django.core.mail.backends.base import BaseEmailBackend
from django.core.mail.message import sanitize_address

from django_aws_mail.signals import mail_pre_send, mail_post_send
from django_aws_mail import settings


class EmailBackend(BaseEmailBackend):
    """
    An email backend for use with Amazon SESv2.
    https://boto3.amazonaws.com/v1/documentation/api/latest/reference/services/sesv2.html

    Overrides the default setting:
    EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'
    """
    def __init__(self,
                 aws_region_name=None,
                 aws_access_key_id=None,
                 aws_secret_access_key=None,
                 fail_silently=False,
                 **kwargs):

        super().__init__(fail_silently=fail_silently, **kwargs)

        self.region_name = aws_region_name or settings.AWS_REGION_NAME
        self.access_key_id = aws_access_key_id or settings.AWS_ACCESS_KEY_ID
        self.secret_access_key = aws_secret_access_key or settings.AWS_SECRET_ACCESS_KEY

        self.connection = None

    def open(self):
        if self.connection:
            return False

        try:
            self.connection = boto3.client('sesv2',
                                           region_name=self.region_name,
                                           aws_access_key_id=self.access_key_id,
                                           aws_secret_access_key=self.secret_access_key)
        except Exception:
            if not self.fail_silently:
                raise

    def close(self):
        self.connection.close()
        self.connection = None

    def send_messages(self, email_messages):
        if not email_messages:
            return 0

        new_conn_created = self.open()
        if not self.connection:
            # fail silently
            return 0

        num_sent = 0
        for message in email_messages:
            sent = self._send(message)
            if sent:
                num_sent += 1

        if new_conn_created:
            self.close()
        return num_sent

    def _send(self, email_message):
        # sending the pre-send signal here allows for changing the message,
        # eg: when removing all recipients (blacklisted) the message will not be sent.
        mail_pre_send.send(self.__class__, message=email_message)

        if not email_message.recipients():
            return False

        encoding = email_message.encoding or settings.DEFAULT_CHARSET
        from_email = sanitize_address(email_message.from_email, encoding)
        recipients = [sanitize_address(addr, encoding) for addr in email_message.recipients()]
        message = email_message.message()

        mail_kwargs = {
            'FromEmailAddress': from_email,
            'Destination': {
                'ToAddresses': recipients,
            },
            'Content': {
                'Raw': {
                    'Data': message.as_bytes(linesep='\r\n'),
                },
            },
        }

        try:
            response = self.connection.send_email(**mail_kwargs)
        except (BotoCoreError, ClientError):
            if not self.fail_silently:
                raise
            return False

        else:
            mail_post_send.send(self.__class__, message=email_message, response=response)

        return True



import re
from email.utils import formataddr

from django.core.mail import EmailMultiAlternatives
from django.template.loader import render_to_string
from django.utils.html import strip_spaces_between_tags

from django_aws_mail import settings
from django_aws_mail.html import HTMLParser


def compose(
        recipients,
        subject,
        html_template,
        text_template=None,
        context=None,
        from_email=None,
        ses_configuration_set=None,
        ses_mail_type_tag=None):

    """
    Create a multipart MIME email message, by rendering html and text body.
    """
    # sanitize input: subject, recipients, from
    subject = ''.join(subject.splitlines())

    if not isinstance(recipients, list):
        recipients = [recipients]

    if not from_email:
        from_email = settings.DEFAULT_FROM_EMAIL
    if not isinstance(from_email, str):
        from_email = formataddr(from_email)

    context = context or {}

    # render content
    html = render_to_string(html_template, context).strip()
    html = strip_spaces_between_tags(html)
    if text_template:
        text = render_to_string(text_template, context).strip()
    else:
        # convert html to text and cleanup
        parser = HTMLParser()
        parser.feed(html)
        parser.close()
        text = parser.text()

    # remove excessive white-space
    text = re.sub(r'( {2,})', '', text)
    text = re.sub(r'(\s{3,})', '\n\n', text)

    # create message
    message = EmailMultiAlternatives(subject, text, from_email, to=recipients)
    message.attach_alternative(html, 'text/html')

    # attach SES specific headers
    if ses_configuration_set:
        message.extra_headers['X-Ses-Configuration-Set'] = ses_configuration_set
    if ses_mail_type_tag:
        message.extra_headers['X-Ses-Message-Tags'] = f'mail-type={ses_mail_type_tag}'

    return message
